@@ -19,36 +19,70 @@
  */
 
 #include <glib.h>
-#include <pulse/pulseaudio.h>
 #include "utils.h"
+#if defined(ELEKTROID_RTAUDIO)
+#include "rtaudio_c.h"
+#else
+#include <pulse/pulseaudio.h>
+#endif
+
+#define AUDIO_CHANNELS 2	// Audio system is always stereo
+
+enum audio_src
+{
+  AUDIO_SRC_NONE,
+  AUDIO_SRC_LOCAL,
+  AUDIO_SRC_REMOTE
+};
+
+enum audio_status
+{
+  AUDIO_STATUS_PREPARING,
+  AUDIO_STATUS_PLAYING,
+  AUDIO_STATUS_STOPPING,
+  AUDIO_STATUS_STOPPED
+};
 
 struct audio
 {
-  GByteArray *sample;
-  guint frames;
-  gboolean loop;
+// PulseAudio or RtAudio backend
+#if defined(ELEKTROID_RTAUDIO)
+  rtaudio_t rtaudio;
+  gdouble volume;
+#else
   pa_threaded_mainloop *mainloop;
   pa_context *context;
   pa_stream *stream;
-  gint pos;
   pa_cvolume volume;
-  uint32_t index;
+  guint32 index;
+#endif
+  GByteArray *sample;
+  guint32 frames;
+  gboolean loop;
+  guint32 pos;
   void (*volume_change_callback) (gdouble);
-  gint release_frames;
-  struct job_control control;
-  gchar *name;
+  guint32 release_frames;
+  struct job_control control;	//Used to synchronize access to sample, frames, loop and pos members.
+  gchar path[PATH_MAX];
+  guint32 channels;
+  enum audio_src src;
+  enum audio_status status;
 };
 
 void audio_play (struct audio *);
 
-void audio_stop (struct audio *, gboolean);
+void audio_stop (struct audio *);
 
 gboolean audio_check (struct audio *);
 
-gint audio_init (struct audio *, void (*)(gdouble), void (*)(gdouble));
+void audio_init (struct audio *, void (*)(gdouble), job_control_callback);
+
+gint audio_run (struct audio *);
 
 void audio_destroy (struct audio *);
 
 void audio_reset_sample (struct audio *);
 
 void audio_set_volume (struct audio *, gdouble);
+
+void audio_write_to_output_buffer (struct audio *, void *, gint);
