@@ -21,6 +21,7 @@
 #include <gtk/gtk.h>
 #include <libgen.h>
 #include "utils.h"
+#include "notifier.h"
 
 #ifndef BROWSER_H
 #define BROWSER_H
@@ -28,23 +29,21 @@
 #define SIZE_LABEL_LEN 16
 
 #define DIR_ICON "folder-visiting-symbolic"
-#define FILE_ICON_WAVE "elektroid-wave-symbolic"
-#define FILE_ICON_DATA "elektroid-data-symbolic"
-#define FILE_ICON_PRJ "elektroid-project-symbolic"
-#define FILE_ICON_SND "elektroid-sound-symbolic"
 
 enum browser_list_field
 {
   BROWSER_LIST_STORE_ICON_FIELD = 0,
-  BROWSER_LIST_STORE_NAME_FIELD,
+  BROWSER_LIST_STORE_NAME_FIELD,	//This is the value returned by the funciton se in the get_item_key member in struct fs_operations. It's the filename.
   BROWSER_LIST_STORE_SIZE_FIELD,
   BROWSER_LIST_STORE_SIZE_STR_FIELD,
   BROWSER_LIST_STORE_TYPE_FIELD,
-  BROWSER_LIST_STORE_INDEX_FIELD
+  BROWSER_LIST_STORE_ID_FIELD,
+  BROWSER_LIST_STORE_SLOT_FIELD	//This is an optional map of the id (number) to some string like "A1", "001" or "[A:001]" to mimic the device way of numbering the items.
 };
 
 struct browser
 {
+  const gchar *name;
   GSourceFunc check_selection;
   GtkTreeView *view;
   GtkWidget *up_button;
@@ -57,30 +56,28 @@ struct browser
   GtkTreePath *dnd_motion_path;
   gint dnd_timeout_function_id;
   GString *dnd_data;
-  void (*notify_dir_change) (struct browser *);
   const gchar *file_icon;
   gchar **extensions;
   const struct fs_operations *fs_ops;
-  void *data;
+  struct backend *backend;
     gboolean (*check_callback) ();
+  struct notifier *notifier;
+  //Background loading members
+  GSList *sensitive_widgets;
+  GtkWidget *stack;
+  GtkWidget *spinner;
+  GThread *thread;
+  GMutex mutex;
+  gboolean loading;
+  gboolean dirty;
 };
 
-gint browser_sort_samples (GtkTreeModel *, GtkTreeIter *, GtkTreeIter *,
-			   gpointer);
-
-gint browser_sort_data (GtkTreeModel *, GtkTreeIter *, GtkTreeIter *,
-			gpointer);
-
-struct item *browser_get_item (GtkTreeModel *, GtkTreeIter *);
-
-void browser_free_item (struct item *);
+void browser_set_item (GtkTreeModel *, GtkTreeIter *, struct item *);
 
 gint browser_get_selected_items_count (struct browser *);
 
 void browser_set_selected_row_iter (struct browser *, GtkTreeIter *);
 
-void browser_reset (gpointer);
-
 void browser_selection_changed (GtkTreeSelection *, gpointer);
 
 void browser_refresh (GtkWidget *, gpointer);
@@ -92,8 +89,20 @@ void browser_item_activated (GtkTreeView *, GtkTreePath *,
 
 gchar *browser_get_item_path (struct browser *, struct item *);
 
-gchar *browser_get_item_id_path (struct browser *, struct item *);
-
 gboolean browser_load_dir (gpointer);
 
+void browser_update_fs_options (struct browser *);
+
+void browser_init (struct browser *);
+
+void browser_destroy (struct browser *);
+
+void browser_set_options (struct browser *);
+
+gboolean browser_set_file_extensions (struct browser *, const gchar **);
+
+gboolean browser_set_file_extension (struct browser *, gchar *);
+
+void browser_reset (struct browser *);
+
 #endif
