@@ -18,26 +18,29 @@
  *   along with Elektroid. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#ifndef NOTIFIER_H
+#define NOTIFIER_H
+
 #include <sys/inotify.h>
 #include <gtk/gtk.h>
 #include "browser.h"
 
 struct notifier
 {
+  gchar *dir;
   gint fd;
   gint wd;
   size_t event_size;
   struct inotify_event *event;
-  gint running;
   struct browser *browser;
+  GThread *thread;
+  GMutex mutex;
 };
 
 void notifier_init (struct notifier *, struct browser *);
 
-void notifier_set_dir (struct notifier *, gchar *);
+void notifier_set_active (struct notifier *, gboolean);
 
-void notifier_close (struct notifier *);
+void notifier_destroy (struct notifier *);
 
-void notifier_free (struct notifier *);
-
-gpointer notifier_run (gpointer);
+#endif
