@@ -28,6 +28,7 @@
 #define PREFERENCES_FILE "/preferences.json"
 
 #define MEMBER_AUTOPLAY "autoplay"
+#define MEMBER_MIX "mix"
 #define MEMBER_LOCALDIR "localDir"
 
 gint
@@ -62,6 +63,9 @@ preferences_save (struct preferences *preferences)
   json_builder_set_member_name (builder, MEMBER_AUTOPLAY);
   json_builder_add_boolean_value (builder, preferences->autoplay);
 
+  json_builder_set_member_name (builder, MEMBER_MIX);
+  json_builder_add_boolean_value (builder, preferences->mix);
+
   json_builder_set_member_name (builder, MEMBER_LOCALDIR);
   json_builder_add_string_value (builder, preferences->local_dir);
 
@@ -86,7 +90,6 @@ preferences_save (struct preferences *preferences)
 gint
 preferences_load (struct preferences *preferences)
 {
-  size_t n;
   GError *error;
   JsonReader *reader;
   JsonParser *parser = json_parser_new ();
@@ -100,7 +103,9 @@ preferences_load (struct preferences *preferences)
 		   CONF_DIR PREFERENCES_FILE, error->message);
       g_error_free (error);
       g_object_unref (parser);
+      g_free (preferences_file);
       preferences->autoplay = TRUE;
+      preferences->mix = TRUE;
       preferences->local_dir = get_expanded_dir ("~");
       return 0;
     }
@@ -117,13 +122,22 @@ preferences_load (struct preferences *preferences)
     }
   json_reader_end_member (reader);
 
-  if (json_reader_read_member (reader, MEMBER_LOCALDIR))
+  if (json_reader_read_member (reader, MEMBER_MIX))
+    {
+      preferences->mix = json_reader_get_boolean_value (reader);
+    }
+  else
+    {
+      preferences->mix = TRUE;
+    }
+  json_reader_end_member (reader);
+
+  if (json_reader_read_member (reader, MEMBER_LOCALDIR) &&
+      g_file_test (json_reader_get_string_value (reader),
+		   (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)))
     {
       preferences->local_dir = malloc (PATH_MAX);
-      n = PATH_MAX - 1;
-      strncpy (preferences->local_dir, json_reader_get_string_value (reader),
-	       n);
-      preferences->local_dir[PATH_MAX - 1] = 0;
+      strcpy (preferences->local_dir, json_reader_get_string_value (reader));
     }
   else
     {
