@@ -18,15 +18,21 @@
  *   along with Elektroid. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "../config.h"
 #include <sndfile.h>
 #include <samplerate.h>
-#include <inttypes.h>
+#include <math.h>
+#include <errno.h>
 #include "sample.h"
 
 #define JUNK_CHUNK_ID "JUNK"
 #define SMPL_CHUNK_ID "smpl"
 
+static const gchar *ELEKTROID_AUDIO_LOCAL_EXTS[] =
+  { "wav", "ogg", "aiff", "flac", NULL };
+
+static const gchar *ELEKTROID_AUDIO_LOCAL_EXTS_MP3[] =
+  { "wav", "ogg", "aiff", "flac", "mp3", NULL };
+
 struct smpl_chunk_data
 {
   guint32 manufacturer;
@@ -62,20 +68,6 @@ static const guint8 JUNK_CHUNK_DATA[] = {
   0, 0, 0, 0
 };
 
-static sf_count_t get_filelen_byte_array_io (void *);
-static sf_count_t seek_byte_array_io (sf_count_t, int, void *);
-static sf_count_t read_byte_array_io (void *, sf_count_t, void *);
-static sf_count_t write_byte_array_io (const void *, sf_count_t, void *);
-static sf_count_t tell_byte_array_io (void *);
-
-static SF_VIRTUAL_IO G_BYTE_ARRAY_IO = {
-  .get_filelen = get_filelen_byte_array_io,
-  .seek = seek_byte_array_io,
-  .read = read_byte_array_io,
-  .write = write_byte_array_io,
-  .tell = tell_byte_array_io
-};
-
 static sf_count_t
 get_filelen_byte_array_io (void *user_data)
 {
@@ -149,10 +141,66 @@ tell_byte_array_io (void *user_data)
   return data->pos;
 }
 
+static SF_VIRTUAL_IO G_BYTE_ARRAY_IO = {
+  .get_filelen = get_filelen_byte_array_io,
+  .seek = seek_byte_array_io,
+  .read = read_byte_array_io,
+  .write = write_byte_array_io,
+  .tell = tell_byte_array_io
+};
+
+static sf_count_t
+get_filelen_file_io (void *user_data)
+{
+  long fileSize, position;
+  FILE *file = user_data;
+  position = ftell (file);
+  fseek (file, 0, SEEK_END);
+  fileSize = ftell (file);
+  fseek (file, position, SEEK_SET);
+  return fileSize;
+}
+
+static sf_count_t
+seek_file_io (sf_count_t offset, int whence, void *user_data)
+{
+  FILE *file = user_data;
+  fseek (file, offset, whence);
+  return ftell (file);
+}
+
+static sf_count_t
+read_file_io (void *ptr, sf_count_t count, void *user_data)
+{
+  FILE *file = user_data;
+  return fread (ptr, 1, count, file);
+}
+
+static sf_count_t
+write_file_io (const void *ptr, sf_count_t count, void *user_data)
+{
+  FILE *file = user_data;
+  return fwrite (ptr, 1, count, file);
+}
+
+static sf_count_t
+tell_file_io (void *user_data)
+{
+  FILE *file = user_data;
+  return ftell (file);
+}
+
+static SF_VIRTUAL_IO FILE_IO = {
+  .get_filelen = get_filelen_file_io,
+  .seek = seek_file_io,
+  .read = read_file_io,
+  .write = write_file_io,
+  .tell = tell_file_io
+};
+
 static gint
-sample_wave_data (GByteArray * sample,
-		  struct job_control *control,
-		  struct g_byte_array_io_data *wave)
+sample_get_wave_data (GByteArray * sample, struct job_control *control,
+		      struct g_byte_array_io_data *wave)
 {
   SF_INFO sf_info;
   SNDFILE *sndfile;
@@ -160,17 +208,20 @@ sample_wave_data (GByteArray * sample,
   struct SF_CHUNK_INFO junk_chunk_info;
   struct SF_CHUNK_INFO smpl_chunk_info;
   struct smpl_chunk_data smpl_chunk_data;
-  struct sample_loop_data *sample_loop_data = control->data;
+  struct sample_info *sample_info = control->data;
 
   g_byte_array_set_size (wave->array, sample->len + 1024);	//We need space for the headers.
   wave->array->len = 0;
 
+  frames = sample->len >> sample_info->channels;
+  debug_print (1, "Frames: %" PRIu64 "; sample rate: %d; channels: %d\n",
+	       frames, sample_info->samplerate, sample_info->channels);
   debug_print (1, "Loop start at %d; loop end at %d\n",
-	       sample_loop_data->start, sample_loop_data->end);
+	       sample_info->loopstart, sample_info->loopend);
 
   memset (&sf_info, 0, sizeof (sf_info));
-  sf_info.samplerate = ELEKTRON_SAMPLE_RATE;
-  sf_info.channels = 1;
+  sf_info.samplerate = sample_info->samplerate;
+  sf_info.channels = sample_info->channels;
   sf_info.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
   sndfile = sf_open_virtual (&G_BYTE_ARRAY_IO, SFM_WRITE, &sf_info, wave);
   if (!sndfile)
@@ -190,7 +241,7 @@ sample_wave_data (GByteArray * sample,
 
   smpl_chunk_data.manufacturer = 0;
   smpl_chunk_data.product = 0;
-  smpl_chunk_data.sample_period = 1000000000 / ELEKTRON_SAMPLE_RATE;
+  smpl_chunk_data.sample_period = 1e9 / sample_info->samplerate;
   smpl_chunk_data.midi_unity_note = 60;
   smpl_chunk_data.midi_pitch_fraction = 0;
   smpl_chunk_data.smpte_format = 0;
@@ -198,9 +249,9 @@ sample_wave_data (GByteArray * sample,
   smpl_chunk_data.num_sampler_loops = 1;
   smpl_chunk_data.sampler_data = 0;
   smpl_chunk_data.sample_loop.cue_point_id = 0;
-  smpl_chunk_data.sample_loop.type = ELEKTRON_LOOP_TYPE;
-  smpl_chunk_data.sample_loop.start = sample_loop_data->start;
-  smpl_chunk_data.sample_loop.end = sample_loop_data->end;
+  smpl_chunk_data.sample_loop.type = sample_info->looptype;
+  smpl_chunk_data.sample_loop.start = sample_info->loopstart;
+  smpl_chunk_data.sample_loop.end = sample_info->loopend;
   smpl_chunk_data.sample_loop.fraction = 0;
   smpl_chunk_data.sample_loop.play_count = 0;
 
@@ -213,15 +264,13 @@ sample_wave_data (GByteArray * sample,
       error_print ("%s\n", sf_strerror (sndfile));
     }
 
-  frames = sample->len >> 1;
-  total = sf_write_short (sndfile, (gint16 *) sample->data, frames);
-
+  total = sf_writef_short (sndfile, (gint16 *) sample->data, frames);
   sf_close (sndfile);
 
   if (total != frames)
     {
-      error_print ("Unexpected frames while writing to sample (%ld != %ld)\n",
-		   total, frames);
+      error_print ("Unexpected frames while writing to file (%" PRIu64 " != %"
+		   PRIu64 ")\n", total, frames);
       return -1;
     }
 
@@ -229,21 +278,21 @@ sample_wave_data (GByteArray * sample,
 }
 
 gint
-sample_wave (GByteArray * sample, GByteArray * wave,
-	     struct job_control *control)
+sample_get_wav_from_array (GByteArray * sample, GByteArray * wave,
+			   struct job_control *control)
 {
   struct g_byte_array_io_data data;
   data.pos = 0;
   data.array = wave;
-  return sample_wave_data (sample, control, &data);
+  return sample_get_wave_data (sample, control, &data);
 }
 
 gint
-sample_save (const gchar * path, GByteArray * sample,
-	     struct job_control *control)
+sample_save_from_array (const gchar * path, GByteArray * sample,
+			struct job_control *control)
 {
   GByteArray *wave = g_byte_array_new ();
-  gint ret = sample_wave (sample, wave, control);
+  gint ret = sample_get_wav_from_array (sample, wave, control);
   if (!ret)
     {
       ret = save_file (path, wave, control);
@@ -272,10 +321,14 @@ audio_multichannel_to_mono (gshort * input, gshort * output, gint size,
     }
 }
 
+// If control->data is NULL, then a new struct sample_info * is created and control->data points to it.
+// In case of failure, if control->data is NULL is freed.
+// Franes is the amount of frames after resampling. This value is set before the loading has terminated.
+
 static gint
-sample_raw_data (struct g_byte_array_io_data *wave,
+sample_load_raw (void *data, SF_VIRTUAL_IO * sf_virtual_io,
 		 struct job_control *control, GByteArray * sample,
-		 guint * frames)
+		 const struct sample_params *sample_params, guint * frames)
 {
   SF_INFO sf_info;
   SNDFILE *sndfile;
@@ -288,12 +341,12 @@ sample_raw_data (struct g_byte_array_io_data *wave,
   gint16 *buffer_input_mono;
   gint16 *buffer_s;
   gfloat *buffer_f;
-  gint err;
-  gint resampled_buffer_len;
-  gint f, frames_read;
+  gint err, resampled_buffer_len, f, frames_read, channels, samplerate;
   gboolean active;
-  struct sample_loop_data *sample_loop_data;
+  gdouble ratio;
+  struct sample_info *sample_info;
   struct smpl_chunk_data smpl_chunk_data;
+  gboolean disable_loop = FALSE;
 
   if (control)
     {
@@ -306,33 +359,88 @@ sample_raw_data (struct g_byte_array_io_data *wave,
     }
 
   sf_info.format = 0;
-  sndfile = sf_open_virtual (&G_BYTE_ARRAY_IO, SFM_READ, &sf_info, wave);
+  sndfile = sf_open_virtual (sf_virtual_io, SFM_READ, &sf_info, data);
   if (!sndfile)
     {
       error_print ("%s\n", sf_strerror (sndfile));
       return -1;
     }
 
+  sample_info = control->data;
+  if (!control->data)
+    {
+      sample_info = g_malloc (sizeof (struct sample_info));
+    }
+
+  channels = sample_params->channels == 2 && sf_info.channels == 2 ? 2 : 1;
+  samplerate =
+    sample_params->samplerate ? sample_params->
+    samplerate : sf_info.samplerate;
+
+  if (control)
+    {
+      g_mutex_lock (&control->mutex);
+    }
+  sample_info->channels = sf_info.channels;
+  sample_info->samplerate = sf_info.samplerate;
+  sample_info->frames = sf_info.frames;
+  if (control)
+    {
+      g_mutex_unlock (&control->mutex);
+    }
+
+  switch (sf_info.format & SF_FORMAT_SUBMASK)
+    {
+    case SF_FORMAT_PCM_S8:
+      sample_info->bitdepth = 8;
+      break;
+    case SF_FORMAT_PCM_16:
+      sample_info->bitdepth = 16;
+      break;
+    case SF_FORMAT_PCM_24:
+      sample_info->bitdepth = 24;
+      break;
+    case SF_FORMAT_PCM_32:
+      sample_info->bitdepth = 32;
+      break;
+    default:
+      sample_info->bitdepth = 0;
+    }
+
   strcpy (chunk_info.id, SMPL_CHUNK_ID);
   chunk_info.id_size = strlen (SMPL_CHUNK_ID);
   chunk_iter = sf_get_chunk_iterator (sndfile, &chunk_info);
-  sample_loop_data = malloc (sizeof (struct sample_loop_data));
+
   if (chunk_iter)
     {
       chunk_info.datalen = sizeof (struct smpl_chunk_data);
+      memset (&smpl_chunk_data, 0, chunk_info.datalen);
       chunk_info.data = &smpl_chunk_data;
       sf_get_chunk_data (chunk_iter, &chunk_info);
-
-      sample_loop_data->start = le32toh (smpl_chunk_data.sample_loop.start);
-      sample_loop_data->end = le32toh (smpl_chunk_data.sample_loop.end);
+      sample_info->loopstart = le32toh (smpl_chunk_data.sample_loop.start);
+      sample_info->loopend = le32toh (smpl_chunk_data.sample_loop.end);
+      sample_info->looptype = le32toh (smpl_chunk_data.sample_loop.type);
+      if (sample_info->loopstart >= sf_info.frames)
+	{
+	  debug_print (2, "Bad loop start\n");
+	  disable_loop = TRUE;
+	}
+      if (sample_info->loopend >= sf_info.frames)
+	{
+	  debug_print (2, "Bad loop end\n");
+	  disable_loop = TRUE;
+	}
     }
   else
     {
-      sample_loop_data->start = 0;
-      sample_loop_data->end = 0;
+      disable_loop = TRUE;
+    }
+  if (disable_loop)
+    {
+      sample_info->loopstart = sf_info.frames - 1;
+      sample_info->loopend = sample_info->loopstart;
+      sample_info->looptype = 0;
     }
-
-  control->data = sample_loop_data;
 
   //Set scale factor. See http://www.mega-nerd.com/libsndfile/api.html#note2
   if ((sf_info.format & SF_FORMAT_FLOAT) == SF_FORMAT_FLOAT ||
@@ -347,38 +455,36 @@ sample_raw_data (struct g_byte_array_io_data *wave,
     malloc (LOAD_BUFFER_LEN * sf_info.channels * sizeof (gint16));
   buffer_input_mono = malloc (LOAD_BUFFER_LEN * sizeof (gint16));
 
-  buffer_f = malloc (LOAD_BUFFER_LEN * sizeof (gfloat));
+  buffer_f = malloc (LOAD_BUFFER_LEN * channels * sizeof (gfloat));
   src_data.data_in = buffer_f;
-  src_data.src_ratio = ((double) ELEKTRON_SAMPLE_RATE) / sf_info.samplerate;
+  ratio = samplerate / (double) sf_info.samplerate;
+  src_data.src_ratio = ratio;
 
-  resampled_buffer_len = LOAD_BUFFER_LEN * src_data.src_ratio;
+  src_data.output_frames = ceil (LOAD_BUFFER_LEN * src_data.src_ratio);
+  resampled_buffer_len = src_data.output_frames * channels;
   buffer_s = malloc (resampled_buffer_len * sizeof (gint16));
   src_data.data_out = malloc (resampled_buffer_len * sizeof (gfloat));
 
-  src_state = src_new (SRC_SINC_BEST_QUALITY, 1, &err);
+  src_state = src_new (SRC_SINC_BEST_QUALITY, channels, &err);
   if (err)
     {
       goto cleanup;
     }
 
-  if (frames)
-    {
-      *frames = sf_info.frames * src_data.src_ratio;
-    }
+  *frames = sf_info.frames * src_data.src_ratio;
 
-  if (ELEKTRON_SAMPLE_RATE != sf_info.samplerate)
+  if (samplerate != sf_info.samplerate)
     {
       debug_print (2, "Loop start at %d, loop end at %d before resampling\n",
-		   sample_loop_data->start, sample_loop_data->end);
-      sample_loop_data->start *= src_data.src_ratio;
-      sample_loop_data->end *= src_data.src_ratio;
+		   sample_info->loopstart, sample_info->loopend);
+      sample_info->loopstart = round (sample_info->loopstart * ratio);
+      sample_info->loopend = round (sample_info->loopend * ratio);
       debug_print (2, "Loop start at %d, loop end at %d after resampling\n",
-		   sample_loop_data->start, sample_loop_data->end);
+		   sample_info->loopstart, sample_info->loopend);
     }
 
   debug_print (2, "Loading sample (%" PRId64 " frames)...\n", sf_info.frames);
 
-  f = 0;
   if (control)
     {
       g_mutex_lock (&control->mutex);
@@ -390,25 +496,15 @@ sample_raw_data (struct g_byte_array_io_data *wave,
       active = TRUE;
     }
 
+  f = 0;
   while (f < sf_info.frames && active)
     {
-      debug_print (2, "Loading buffer...\n");
-
-      frames_read =
-	sf_readf_short (sndfile, buffer_input_multi, LOAD_BUFFER_LEN);
-
-      if (frames_read < LOAD_BUFFER_LEN)
-	{
-	  src_data.end_of_input = SF_TRUE;
-	}
-      else
-	{
-	  src_data.end_of_input = 0;
-	}
-      src_data.input_frames = frames_read;
+      debug_print (2, "Loading %d channels buffer...\n", channels);
+      frames_read = sf_readf_short (sndfile, buffer_input_multi,
+				    LOAD_BUFFER_LEN);
       f += frames_read;
 
-      if (sf_info.channels == 1)
+      if (channels == sf_info.channels)	// 1 <= channels <= 2
 	{
 	  buffer_input = buffer_input_multi;
 	}
@@ -419,14 +515,14 @@ sample_raw_data (struct g_byte_array_io_data *wave,
 	  buffer_input = buffer_input_mono;
 	}
 
-      if (sf_info.samplerate == ELEKTRON_SAMPLE_RATE)
+      if (samplerate == sf_info.samplerate)
 	{
 	  if (control)
 	    {
 	      g_mutex_lock (&control->mutex);
 	    }
 	  g_byte_array_append (sample, (guint8 *) buffer_input,
-			       frames_read << 1);
+			       frames_read << channels);
 	  if (control)
 	    {
 	      g_mutex_unlock (&control->mutex);
@@ -434,23 +530,29 @@ sample_raw_data (struct g_byte_array_io_data *wave,
 	}
       else
 	{
-	  src_short_to_float_array (buffer_input, buffer_f, frames_read);
-	  src_data.output_frames = src_data.input_frames * src_data.src_ratio;
+	  src_data.end_of_input = frames_read < LOAD_BUFFER_LEN ? SF_TRUE : 0;
+	  src_data.input_frames = frames_read;
+
+	  src_short_to_float_array (buffer_input, buffer_f,
+				    frames_read * channels);
+	  debug_print (2, "Resampling %d channels with ratio %f...\n",
+		       channels, src_data.src_ratio);
 	  err = src_process (src_state, &src_data);
-	  debug_print (2, "Resampling...\n");
 	  if (err)
 	    {
-	      debug_print (2, "Error %s\n", src_strerror (err));
+	      g_byte_array_set_size (sample, 0);
+	      error_print ("Error while resampling: %s\n",
+			   src_strerror (err));
 	      break;
 	    }
 	  src_float_to_short_array (src_data.data_out, buffer_s,
-				    src_data.output_frames_gen);
+				    src_data.output_frames_gen * channels);
 	  if (control)
 	    {
 	      g_mutex_lock (&control->mutex);
 	    }
 	  g_byte_array_append (sample, (guint8 *) buffer_s,
-			       src_data.output_frames_gen << 1);
+			       src_data.output_frames_gen << channels);
 	  if (control)
 	    {
 	      g_mutex_unlock (&control->mutex);
@@ -459,8 +561,9 @@ sample_raw_data (struct g_byte_array_io_data *wave,
 
       if (control)
 	{
-	  control->callback (f * 1.0 / sf_info.frames);
 	  g_mutex_lock (&control->mutex);
+	  set_job_control_progress_no_sync (control,
+					    f * 1.0 / sf_info.frames);
 	  active = control->active;
 	  g_mutex_unlock (&control->mutex);
 	}
@@ -468,19 +571,13 @@ sample_raw_data (struct g_byte_array_io_data *wave,
 
   src_delete (src_state);
 
-  if (err)
-    {
-      error_print ("Error while preparing resampling: %s\n",
-		   src_strerror (err));
-    }
-
   if (control)
     {
       g_mutex_lock (&control->mutex);
 
       if (control->active)
 	{
-	  control->callback (1.0);
+	  set_job_control_progress_no_sync (control, 1.0);
 	}
       else
 	{
@@ -499,56 +596,57 @@ cleanup:
 
   sf_close (sndfile);
 
-  if (!sample->len)
+  if (sample->len)
     {
-      g_free (control->data);
+      if (!control->data)
+	{
+	  control->data = sample_info;
+	}
+      // This removes the additional samples added by the resampler due to rounding.
+      g_byte_array_set_size (sample, *frames << channels);
+      return 0;
+    }
+  else
+    {
+      if (!control->data)
+	{
+	  g_free (sample_info);
+	}
+      return -1;
     }
-
-  return sample->len > 0 ? 0 : -1;
 }
 
-static gint
-sample_raw_frames (GByteArray * wave, GByteArray * sample,
-		   struct job_control *control, guint * frames)
+gint
+sample_load_from_array (GByteArray * wave, GByteArray * sample,
+			struct job_control *control,
+			const struct sample_params *sample_params,
+			guint * frames)
 {
   struct g_byte_array_io_data data;
   data.pos = 0;
   data.array = wave;
-  return sample_raw_data (&data, control, sample, frames);
+  return sample_load_raw (&data, &G_BYTE_ARRAY_IO, control, sample,
+			  sample_params, frames);
 }
 
 gint
-sample_raw (GByteArray * wave, GByteArray * sample,
-	    struct job_control *control)
+sample_load_from_file (const gchar * path, GByteArray * sample,
+		       struct job_control *control,
+		       const struct sample_params *sample_params,
+		       guint * frames)
 {
-  return sample_raw_frames (wave, sample, control, NULL);
-}
-
-gint
-sample_load_with_frames (const gchar * path, GByteArray * sample,
-			 struct job_control *control, guint * frames)
-{
-  GByteArray *wave = g_byte_array_new ();
-
-  debug_print (1, "Loading file %s...\n", path);
-
-  int ret = load_file (path, wave, control);
-  if (!ret)
+  FILE *file = fopen (path, "rb");
+  if (!file)
     {
-      ret = sample_raw_frames (wave, sample, control, frames);
+      return errno;
     }
-  g_byte_array_free (wave, TRUE);
-  return ret;
+  gint err = sample_load_raw (file, &FILE_IO, control, sample, sample_params,
+			      frames);
+  fclose (file);
+  return err;
 }
 
-gint
-sample_load (const gchar * path, GByteArray * sample,
-	     struct job_control *control)
-{
-  return sample_load_with_frames (path, sample, control, NULL);
-}
-
-gboolean
+static gboolean
 sample_is_mp3_supported ()
 {
   static char buffer[LABEL_MAX];
@@ -562,3 +660,13 @@ sample_is_mp3_supported ()
 
   return FALSE;
 }
+
+const gchar **
+sample_get_sample_extensions ()
+{
+  if (sample_is_mp3_supported ())
+    {
+      return ELEKTROID_AUDIO_LOCAL_EXTS_MP3;
+    }
+  return ELEKTROID_AUDIO_LOCAL_EXTS;
+}
