@@ -26,19 +26,20 @@
 #ifndef SAMPLE_H
 #define SAMPLE_H
 
-#define LOAD_BUFFER_LEN 4800	// In guint16 frames; 9600 B; 0.1 ms
+#define LOAD_BUFFER_LEN 9600	// In guint16 frames; 0.2 ms
 
-gint sample_wave (GByteArray *, GByteArray *, struct job_control *);
+gint sample_get_wav_from_array (GByteArray *, GByteArray *,
+				struct job_control *);
 
-gint sample_raw (GByteArray *, GByteArray *, struct job_control *);
+gint sample_save_from_array (const gchar *, GByteArray *,
+			     struct job_control *);
 
-gint sample_save (const gchar *, GByteArray *, struct job_control *);
+gint sample_load_from_array (GByteArray *, GByteArray *, struct job_control *,
+			     const struct sample_params *, guint *);
 
-gint sample_load (const gchar *, GByteArray *, struct job_control *);
+gint sample_load_from_file (const gchar *, GByteArray *, struct job_control *,
+			    const struct sample_params *, guint *);
 
-gint sample_load_with_frames (const gchar *, GByteArray *,
-			      struct job_control *, guint *);
-
-gboolean sample_is_mp3_supported ();
+const gchar **sample_get_sample_extensions ();
 
 #endif
