@@ -24,18 +24,10 @@
 #include "utils.h"
 
 #define DEBUG_SHORT_HEX_LEN 64
-#define DEBUG_FULL_HEX_THRES 3
+#define DEBUG_FULL_HEX_THRES 5
 
 #define KIB 1024
 
-#define DEVICES_FILE "/devices.json"
-
-#define DEV_TAG_ID "id"
-#define DEV_TAG_NAME "name"
-#define DEV_TAG_ALIAS "alias"
-#define DEV_TAG_FILESYSTEMS "filesystems"
-#define DEV_TAG_STORAGE "storage"
-
 gint debug_level;
 
 static guint
@@ -138,15 +130,19 @@ void
 remove_ext (char *name)
 {
   gint namelen = strlen (name);
-  gchar *dot = &name[namelen];
-  gint i = namelen;
+  gchar *dot = &name[namelen - 1];
+  gint i = namelen - 1;
 
-  while (*dot != '.' && i > 0)
+  while (i > 0)
     {
+      if (*dot == '.')
+	{
+	  *dot = 0;
+	  break;
+	}
       dot--;
       i--;
     }
-  *dot = 0;
 }
 
 const gchar *
@@ -225,29 +221,17 @@ free_msg (gpointer msg)
 }
 
 gchar *
-get_item_name (struct item *item)
+get_filename (guint32 fs_options, struct item *item)
 {
+  if (fs_options & FS_OPTION_ID_AS_FILENAME && item->type == ELEKTROID_FILE)
+    {
+      gchar *id = malloc (LABEL_MAX);
+      snprintf (id, LABEL_MAX, "%d", item->id);
+      return id;
+    }
   return strdup (item->name);
 }
 
-gchar *
-get_item_index (struct item *item)
-{
-  gchar *index;
-
-  if (item->type == ELEKTROID_DIR)
-    {
-      index = get_item_name (item);
-    }
-  else
-    {
-      index = malloc (LABEL_MAX);
-      snprintf (index, LABEL_MAX, "%d", item->index);
-    }
-
-  return index;
-}
-
 guint
 next_item_iterator (struct item_iterator *iter)
 {
@@ -263,18 +247,11 @@ free_item_iterator (struct item_iterator *iter)
     }
 }
 
-gint
-copy_item_iterator (struct item_iterator *dst, struct item_iterator *src,
-		    gboolean cached)
-{
-  return src->copy (dst, src, cached);
-}
-
 gint
 load_file (const char *path, GByteArray * array, struct job_control *control)
 {
   FILE *file;
-  long size;
+  size_t size;
   gint res;
 
   file = fopen (path, "rb");
@@ -300,7 +277,7 @@ load_file (const char *path, GByteArray * array, struct job_control *control)
 
   if (fread (array->data, 1, size, file) == size)
     {
-      debug_print (1, "%zu bytes read\n", size);
+      debug_print (1, "%zu B read\n", size);
     }
   else
     {
@@ -317,7 +294,7 @@ gint
 save_file_char (const gchar * path, const guint8 * data, ssize_t len)
 {
   gint res;
-  long bytes;
+  size_t bytes;
   FILE *file;
 
   file = fopen (path, "w");
@@ -333,7 +310,7 @@ save_file_char (const gchar * path, const guint8 * data, ssize_t len)
   bytes = fwrite (data, 1, len, file);
   if (bytes == len)
     {
-      debug_print (1, "%zu bytes written\n", bytes);
+      debug_print (1, "%zu B written\n", bytes);
     }
   else
     {
@@ -354,14 +331,18 @@ save_file (const gchar * path, GByteArray * array,
 }
 
 gchar *
-get_human_size (guint size, gboolean with_space)
+get_human_size (gint64 size, gboolean with_space)
 {
   gchar *label = malloc (LABEL_MAX);
   gchar *space = with_space ? " " : "";
 
-  if (size < KIB)
+  if (size < 0)
     {
-      snprintf (label, LABEL_MAX, "%d%sB", size, space);
+      *label = 0;
+    }
+  else if (size < KIB)
+    {
+      snprintf (label, LABEL_MAX, "%" PRId64 "%sB", size, space);
     }
   else if (size < KIB * KIB)
     {
@@ -381,162 +362,73 @@ get_human_size (guint size, gboolean with_space)
   return label;
 }
 
+static inline void
+set_job_control_progress_value (struct job_control *control, gdouble p)
+{
+  control->progress =
+    (control->part / (double) control->parts) + (p / (double) control->parts);
+}
+
 void
 set_job_control_progress (struct job_control *control, gdouble p)
 {
-  gdouble v = (double) control->part / control->parts + p / control->parts;
-  control->callback (v);
+  g_mutex_lock (&control->mutex);
+  set_job_control_progress_value (control, p);
+  g_mutex_unlock (&control->mutex);
+
+  if (control->callback)
+    {
+      control->callback (control);
+    }
 }
 
-gint
-load_device_desc (struct connector_device_desc *device_desc, guint8 id,
-		  const char *devices_filename_)
+void
+set_job_control_progress_no_sync (struct job_control *control, gdouble p)
 {
-  gint err, devices;
-  JsonParser *parser;
-  JsonReader *reader;
-  gchar *devices_filename;
-  GError *error = NULL;
+  set_job_control_progress_value (control, p);
 
-  parser = json_parser_new ();
-
-  if (devices_filename_)
+  if (control->callback)
     {
-      devices_filename = strdup (devices_filename_);
-
-      if (!json_parser_load_from_file (parser, devices_filename, &error))
-	{
-	  error_print ("%s", error->message);
-	  g_clear_error (&error);
-	  err = -ENODEV;
-	  goto cleanup_parser;
-	}
+      control->callback (control);
     }
-  else
-    {
-      devices_filename = get_expanded_dir (CONF_DIR DEVICES_FILE);
-
-      if (!json_parser_load_from_file (parser, devices_filename, &error))
-	{
-	  debug_print (1, "%s\n", error->message);
-	  g_clear_error (&error);
-
-	  g_free (devices_filename);
-	  devices_filename = strdup (DATADIR DEVICES_FILE);
+}
 
-	  debug_print (1, "Falling back to %s...\n", devices_filename);
+gboolean
+file_matches_extensions (const gchar * name, gchar ** extensions)
+{
+  const gchar *extension;
+  gchar **e = extensions;
 
-	  if (!json_parser_load_from_file (parser, devices_filename, &error))
-	    {
-	      error_print ("%s", error->message);
-	      g_clear_error (&error);
-	      err = -ENODEV;
-	      goto cleanup_parser;
-	    }
-	}
-    }
-
-  reader = json_reader_new (json_parser_get_root (parser));
-  if (!reader)
+  if (!e)
     {
-      error_print ("Unable to read from parser");
-      err = -ENODEV;
-      goto cleanup_parser;
+      return TRUE;
     }
 
-  if (!json_reader_is_array (reader))
+  extension = get_ext (name);
+  if (!extension)
     {
-      error_print ("Not an array\n");
-      err = -ENODEV;
-      goto cleanup_reader;
+      return FALSE;
     }
 
-  devices = json_reader_count_elements (reader);
-  if (!devices)
+  while (*e)
     {
-      debug_print (1, "No devices found\n");
-      err = -ENODEV;
-      goto cleanup_reader;
-    }
-
-  err = -ENODEV;
-  for (int i = 0; i < devices; i++)
-    {
-      if (!json_reader_read_element (reader, i))
+      if (!strcasecmp (extension, *e))
 	{
-	  error_print ("Cannot read element %d. Continuing...\n", i);
-	  continue;
+	  return TRUE;
 	}
+      e++;
+    }
 
-      if (!json_reader_read_member (reader, DEV_TAG_ID))
-	{
-	  error_print ("Cannot read member '%s'. Continuing...\n",
-		       DEV_TAG_ID);
-	  continue;
-	}
-      device_desc->id = json_reader_get_int_value (reader);
-      json_reader_end_member (reader);
+  return FALSE;
+}
 
-      if (device_desc->id != id)
-	{
-	  json_reader_end_element (reader);
-	  continue;
-	}
-
-      err = 0;
-      debug_print (1, "Device %d found\n", id);
-
-      if (!json_reader_read_member (reader, DEV_TAG_NAME))
-	{
-	  error_print ("Cannot read member '%s'. Stopping...\n",
-		       DEV_TAG_NAME);
-	  json_reader_end_element (reader);
-	  err = -ENODEV;
-	  break;
-	}
-      device_desc->name = strdup (json_reader_get_string_value (reader));
-      json_reader_end_member (reader);
-
-      if (!json_reader_read_member (reader, DEV_TAG_ALIAS))
-	{
-	  error_print ("Cannot read member '%s'. Stopping...\n",
-		       DEV_TAG_ALIAS);
-	  json_reader_end_element (reader);
-	  err = -ENODEV;
-	  break;
-	}
-      device_desc->alias = strdup (json_reader_get_string_value (reader));
-      json_reader_end_member (reader);
-
-      if (!json_reader_read_member (reader, DEV_TAG_FILESYSTEMS))
-	{
-	  error_print ("Cannot read member '%s'. Stopping...\n",
-		       DEV_TAG_FILESYSTEMS);
-	  json_reader_end_element (reader);
-	  err = -ENODEV;
-	  break;
-	}
-      device_desc->filesystems = json_reader_get_int_value (reader);
-      json_reader_end_member (reader);
-
-      if (!json_reader_read_member (reader, DEV_TAG_STORAGE))
-	{
-	  error_print ("Cannot read member '%s'. Stopping...\n",
-		       DEV_TAG_STORAGE);
-	  json_reader_end_element (reader);
-	  err = -ENODEV;
-	  break;
-	}
-      device_desc->storage = json_reader_get_int_value (reader);
-      json_reader_end_member (reader);
-
-      break;
+gboolean
+iter_matches_extensions (struct item_iterator *iter, gchar ** extensions)
+{
+  if (iter->item.type == ELEKTROID_DIR)
+    {
+      return TRUE;
     }
 
-cleanup_reader:
-  g_object_unref (reader);
-cleanup_parser:
-  g_object_unref (parser);
-  g_free (devices_filename);
-  return err;
+  return file_matches_extensions (iter->item.name, extensions);
 }
